@@ -4,4 +4,4 @@
 from atypes.util import MyType
 from atypes.typ import *
 
-from atypes.protocols import Gettable
+from atypes.protocols import Gettable, Sliceable
