@@ -1,6 +1,18 @@
 """Types and protocols"""
 
-from typing import Callable, Union, Any, List, Tuple, Iterable, Sequence
+from typing import (
+    Callable,
+    Any,
+    List,
+    Tuple,
+    Iterable,
+    Sequence,
+    Mapping,
+    TypeVar,
+    T,
+    KT,
+    MutableMapping,
+)
 from numbers import Number
 
 # from numpy import ndarray, int16, int32, float32, float64
@@ -14,6 +26,82 @@ from atypes.util import MyType
 # from functools import partial
 # NT = partial(new_type, assign_to_globals=True)
 
+# ------------------ GENERAL ------------------------------------------------------------
+
+Factory = Callable[..., T]
+Factory.__doc__ = 'A function that makes objects of a specific kind'
+
+Store = Mapping[Any, T]
+Store.__doc__ = 'A mapping-interface to a store (usually persistent) of data'
+
+WritableStore = MutableMapping[Any, T]
+WritableStore.__doc__ = (
+    'A (mutuable-)mapping-interface to a store of data that allows writing to it'
+)
+# ------------------ ML -----------------------------------------------------------------
+
+
+Feature = MyType(
+    'Feature',
+    Number,  # if categorical needs to be cast to number (as sklearn obliges)
+    doc='A number that represents a characteristic of something. '
+    'Usually appears as an item of an FV (a sequence of Features)',
+)
+# FV = FixedSizeSeq[Feature]
+
+FV = MyType(
+    'FV',
+    Sequence[Feature],
+    doc='Feature Vector. The informational fingerprint of something.',
+)
+FVs = MyType('FVs', Iterable[FV], aka=['fvs'], doc='An iterable of FVs')
+
+Featurizer = MyType(
+    'Featurizer',
+    Callable[[Any], FV],
+    doc='A function that makes FVs (out of Chunks, other FVs, or anything really. '
+    '(This is a declaration that the output will be FVs, not what the input should be.)',
+)
+
+# from typing import Protocol
+#
+#
+# Data = Any
+#
+# ModelFunc = Callable
+# Learner = Any
+# Fitter = Callable[[Learner, Data, ...], ModelFunc]
+#
+# Xtype = TypeVar('Xtype')
+# Ytype = TypeVar('Ytype')
+#
+# # SupervisedFitter = Callable[[Xtype, Ytype, ...], Callable[[Xtype], Ytype]]
+# # didn't work, so:
+# class SupervisedFitter(Protocol):
+#     """Describes a supervised fitting function that takes some (X, y) pair of
+#     iterables and returns a callable that takes an X and returns a y.
+#     The X is meant to describe an array of feature vectors and y a corresponding
+#     array of values associated with each vector.
+#     A supervised fitter takes a (X, y) pair and returns a function whose role it is
+#     to take X inputs and return estimations/predictions of what y would be.
+#     """
+#
+#     def __call__(
+#         # note, using double underscores here to indicate that the name of the
+#         # argument shouldn't matter
+#         # See: https://mypy.readthedocs.io/en/stable/protocols.html#callback-protocols
+#         self, __X: Xtype, __y: Ytype, *args, **kwargs
+#     ) -> Callable[[Xtype], Ytype]:
+#         pass
+
+
+# SupervisedFitter = Callable[[FVs, Iterable[TargetType]], ModelFunc]
+# Fitter = Callable[[Learner, Data, ...], ModelFunc]
+
+# SupervisedData = Tuple[Fv]
+
+
+# ------------------ SIGNAL ML ----------------------------------------------------------
 FiltFunc = MyType(
     'FiltFunc',
     Callable[[Any], bool],
@@ -50,29 +138,9 @@ Chunker = MyType(
     'Chunker',
     Callable[[Waveform], Iterable[Chunk]],
     aka=['chunker', 'wf_to_chks'],
-    doc='The component that generates Chunks from a Waveform',
+    doc='A callable that generates Chunks from a Waveform',
 )
 
-Feature = MyType(
-    'Feature',
-    Number,
-    doc='A number that represents a characteristic of something. '
-    'Usually appears as an item of an FV (a sequence of Features)',
-)
-# FV = FixedSizeSeq[Feature]
-
-FV = MyType(
-    'FV',
-    Sequence[Feature],
-    doc='Feature Vector. The informational fingerprint of something.',
-)
-FVs = MyType('FVs', Iterable[FV], aka=['fvs'], doc='An iterable of FVs')
-Featurizer = MyType(
-    'Featurizer',
-    Callable[[Any], FV],
-    doc='A function that makes FVs (out of Chunks, other FVs, or anything really. '
-    '(This is a declaration that the output will be FVs, not what the input should be.)',
-)
 ChkFeaturizer = MyType(
     'ChkFeaturizer',
     Callable[[Chunk], FV],
@@ -142,6 +210,17 @@ Hunk = MyType(
 
 WaveformBytes = MyType('WaveformBytes', bytes)
 
+# --------------- STORES ----------------------------------------------------------------
+
+# WfStore = StoreType[Waveform]
+
+WfStore = MyType(
+    'WfStore',
+    Store[Waveform],
+    aka=['wf_store', 'wfs', 'audio_store'],
+    doc='A waveform store. More precisely, a key-value (Mapping) interface to waveforms',
+)
+
 # --------------- SLANG TYPES -----------------------------------------------------------
 
 # ChkFeaturizer
